//
//  BLHelper.m
//  TestC
//
//  Created by long yang on 2021/9/27.
//  Copyright © 2021 long yang. All rights reserved.
//

#import "BLHelper.h"
#import <CoreBluetooth/CoreBluetooth.h>
#import <UIKit/UIKit.h>
#import <DBAlgorithm/DBAlgorithm.h>
//#include "UnityInterface.h"

#define SERVICE_NAME @"Dbay"
#define DEVICE_SERVIVE_UUID  @"DBA8"
#define SERVICE_UUID  @"6E400001-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC1_UUID  @"6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC2_UUID  @"6E400003-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC3_UUID  @"6E400004-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC4_UUID  @"6E400005-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC5_UUID  @"6E400006-B5A3-F393-E0A9-E50E24DCCA9E"

typedef NS_ENUM(NSInteger, TYPE) {
    type_none = 0, // 默认没有
    type_data = 1, // 设备算法指标返回
    type_find_device = 2,// 搜索设备返回
    type_connect_succeed = 3,// 连接设备成功返回
    type_connect_failed = 4, // 连接设备失败返回
    type_disconnected = 5,// 断开设备回调
    type_get_device_info =6, // 获取设备信息回调
};

@interface BLHelper()<CBCentralManagerDelegate,CBPeripheralDelegate, CBPeripheralManagerDelegate>
{
    NSMutableArray * _allDevice;// 存储所有已经搜索到的设备
    NSMutableArray * _allConnectedDevice;// 存储所有已经连接的设备
}

@property (nonatomic, strong)CBCentralManager * centerManager; //声明属性存储中心设备
@property (nonatomic, copy)NSMutableDictionary * dataCache; // 存储每个设备所有的原始数据，传入到unity方面
@property (nonatomic, copy)NSMutableDictionary * rawInfo; // 存储每个设备是否打开raw原数据的开关
@property (nonatomic, copy)NSMutableDictionary * notifyState; // 存储设备的信息，如电量，版本等

@end

static NSString *receiver = @"BLHelper";
static NSString *method = @"onMsg";

@implementation BLHelper

+(BLHelper *)sharedInstance
{
    __strong static BLHelper * _instance=nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _instance = [[super alloc] init:nil];
    });
    return _instance;
}

-(id)init:(NSString *)param
{
    if(self=[super init]){
        if(param&&[param length]>0){
            NSData * _data = [param dataUsingEncoding:NSUTF8StringEncoding];
            NSDictionary * _dict = [NSJSONSerialization JSONObjectWithData:_data options:NSJSONReadingMutableContainers error:NULL];
            if([_dict objectForKey:@"blReceiver"]){
                receiver=[_dict objectForKey:@"blReceiver"];
            }
            if([_dict objectForKey:@"blMethod"]){
                method=[_dict objectForKey:@"blMethod"];
            }
        }

        // 初始化SDK
        [[DBAlgorithmCalculate sharedInstance] startWithToken:@"c2dc8da8-7h66-4w04-a237-c705e28824de" accountId:@"6176085114a58df902ab0587" completion:^(DBErrorCode code) {
            if(code==DBErrorCodeSuccess){
                NSLog(@"可以成功使用SDK");
            } else {
                NSLog(@"不能使用SDK");
            }
        }];

        // 初始化本地全局变量
        _allDevice=[NSMutableArray new];
        _allConnectedDevice=[NSMutableArray new];
        _notifyState=[NSMutableDictionary new];
        _dataCache=[NSMutableDictionary new];
        _rawInfo=[NSMutableDictionary new];
        _centerManager=[[CBCentralManager alloc] initWithDelegate:self queue:dispatch_get_main_queue()];
    }
    return self;
}

/**
 获取手机蓝牙状态
 */
-(BOOL)isAvailable
{
    return self.centerManager.state == CBManagerStatePoweredOn;
}

/**
 扫描搜索头环设备
 */
-(void)scanDevice
{
    [_centerManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:DEVICE_SERVIVE_UUID]] options:@{CBCentralManagerScanOptionAllowDuplicatesKey:@(NO)}];
}

/**
 停止搜索头环设备
 */
-(void)stopScan
{
    [_centerManager stopScan];
}

/**
 获取设备的电量
 */
-(void)getVersion:(NSString *)address
{
    NSDictionary * _dict=[_notifyState objectForKey:address];
    
    CBPeripheral * _peripheral=[_dict objectForKey:@"peripheral"];
    CBCharacteristic * _character=[_dict objectForKey:@"character"];
    // 获取设备版本信息
    Byte versionByte[]={0x01};
    NSData * _data=[[NSData alloc] initWithBytes:versionByte length:sizeof(versionByte)];
    [_peripheral writeValue:_data forCharacteristic:_character type:CBCharacteristicWriteWithResponse];
}

/**
 获取设备的版本
 */
-(void)getBattery:(NSString *)address
{
    NSDictionary * _dict=[_notifyState objectForKey:address];
    
    CBPeripheral * _peripheral=[_dict objectForKey:@"peripheral"];
    CBCharacteristic * _character=[_dict objectForKey:@"character"];

    // 获取设备电量信息
    Byte batteryByte[]={0x02};
    NSData * _batteryData=[[NSData alloc] initWithBytes:batteryByte length:sizeof(batteryByte)];
    [_peripheral writeValue:_batteryData forCharacteristic:_character type:CBCharacteristicWriteWithResponse];
}

/**
 获取设备信息
 */
-(void)getDevice:(NSString *)address
{
    [self getBattery:address];
    [self getVersion:address];
}

/**
 连接某台头环设备
 */
-(void)connectDevice:(NSString *)address
{
    for (int i = 0; i < _allDevice.count; i++) {
        CBPeripheral * per = _allDevice[i];
        if ([per.identifier.UUIDString isEqualToString:address]) {
            if(per.state == CBPeripheralStateConnecting){
                NSLog(@"设备正在连接中");
                return;
            }
            if(per.state == CBPeripheralStateConnected){
                NSLog(@"设备已连接，请勿重复连接");
                return;
            }
            
            [_centerManager connectPeripheral:per options:nil];
            NSLog(@"BLHelper---isConnecting %@", per.identifier.UUIDString);
            return;
        }
    }
}

/**
 断开当前连接的头环设备
 */
-(void)disconnectDevice:(NSString *)address
{
    for (int i = 0; i < _allDevice.count; i++) {
        CBPeripheral * peripheral= _allDevice[i];
        if ([peripheral.identifier.UUIDString isEqualToString:address]) {
            if(peripheral.state != CBPeripheralStateConnected){
                NSLog(@"设备未处于连接状态");
                return;
            }
            [self.centerManager cancelPeripheralConnection:peripheral];
            return;
        }
    }
}

/**
 传入设置是否打开原始数据的开关
 */
-(void)openWriteRaw:(NSString *)address
{
    [_rawInfo setObject:[NSNumber numberWithBool:YES] forKey:address];
}

#pragma mark -- 公用方法

NSString* getString(char* str)
{
    return [NSString stringWithUTF8String:str];
}

const char* getChar(NSString* str)
{
    return [str cStringUsingEncoding:NSUTF8StringEncoding];
}

-(NSString*)objectToJson:(id)obj
{
    if([NSJSONSerialization isValidJSONObject:obj]){
        NSData *data = [NSJSONSerialization dataWithJSONObject:obj options:NSJSONReadingAllowFragments error:nil];
        return [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    }
    return @"";
}

-(void)sendToApp:(NSString*)json
{
    // PS:当使用unity3d的时候，使用下面的方法发送消息，同时需要打开"UnityInterface.h"文件的引用
    UnitySendMessage(getChar(receiver), getChar(method), getChar(json));
}

-(void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    NSLog(@"%@",central);
    switch (central.state) {
        case CBManagerStatePoweredOn:
            NSLog(@"打开，可用");
            break;
        case CBManagerStatePoweredOff:
            NSLog(@"可用，未打开");
            break;
        case CBManagerStateUnsupported:
            NSLog(@"SDK不支持");
            break;
        case CBManagerStateUnauthorized:
            NSLog(@"程序未授权");
            break;
        case CBManagerStateResetting:
            NSLog(@"CBCentralManagerStateResetting");
            break;
        case CBManagerStateUnknown:
            NSLog(@"CBCentralManagerStateUnknown");
            break;
    }
}

- (void)peripheralManagerDidUpdateState:(nonnull CBPeripheralManager *)peripheral
{
    
}

#pragma mark -- 代理方法

- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary<NSString *, id> *)advertisementData RSSI:(NSNumber *)RSSI
{
    if (peripheral.name.length <= 0) {
        return ;
    }
    
    if([peripheral.name hasPrefix:SERVICE_NAME]){
        [_allDevice addObject:peripheral];
        
        NSDictionary * deviceDict = @{
            @"address": peripheral.identifier.UUIDString,
            @"name": peripheral.name,
            @"state": [NSNumber numberWithInt:(int)peripheral.state],
            @"type": [NSNumber numberWithInt:type_find_device],
        };
        [self sendToApp:[self objectToJson:deviceDict]];
    }
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSMutableDictionary * _dict=[NSMutableDictionary new];
    [_dict setObject:peripheral forKey:@"peripheral"];
    int index=[[DBAlgorithmCalculate sharedInstance] addDevice:peripheral.name];
    [_dict setObject:[NSString stringWithFormat:@"%d",index] forKey:@"index"];
    [_allConnectedDevice addObject:_dict];
    peripheral.delegate=self;
    [peripheral discoverServices:@[[CBUUID UUIDWithString:SERVICE_UUID]]];
    
    //PS：需要在此处发送消息给unity3d，告诉他们的设备信息
    NSDictionary * deviceDict = @{
        @"address": peripheral.identifier.UUIDString,
        @"name": peripheral.name,
        @"state": [NSNumber numberWithInt:(int)peripheral.state],
        @"type":[NSNumber numberWithInt:type_connect_succeed],
    };
    [self sendToApp:[self objectToJson:deviceDict]];
}

- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(nullable NSError *)error
{
    NSLog(@"didFailToConnectPeripheral");
    NSDictionary * deviceDict = @{
        @"address": peripheral.identifier.UUIDString,
        @"name": peripheral.name,
        @"type":[NSNumber numberWithInt:type_connect_failed],
        @"state": [NSNumber numberWithInt:(int)peripheral.state],
    };
    [self sendToApp:[self objectToJson:deviceDict]];
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(nullable NSError *)error
{
    NSString *UUID = [peripheral.identifier UUIDString];
    NSLog(@"已经搜索到了设备:%@",UUID);
    if (error) {
        NSLog(@"出错");
        return;
    }
    
    CBUUID *cbUUID = [CBUUID UUIDWithString:UUID];
    NSLog(@"cbUUID:%@",cbUUID);
    
    CBService * _service=peripheral.services.lastObject;
    if(_service){
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC1_UUID],[CBUUID UUIDWithString:CHARACTERISTIC2_UUID],[CBUUID UUIDWithString:CHARACTERISTIC3_UUID],[CBUUID UUIDWithString:CHARACTERISTIC4_UUID],[CBUUID UUIDWithString:CHARACTERISTIC5_UUID]] forService:_service];
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(nullable NSError *)error
{
    if (error) {
        NSLog(@"出错");
        return;
    }
    
    for (CBCharacteristic *character in service.characteristics) {
        if([character.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC1_UUID]]){
            if([_notifyState objectForKey:peripheral.identifier.UUIDString]==nil){
                __weak __typeof(self) _weakSelf = self;
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                
                    // 打开数据通道
                    NSData * _openData=nil;
                    if([peripheral.name hasPrefix:@"Dbay-EEG2"]||[peripheral.name hasPrefix:@"Dbay-EEGM"]){
                        Byte byte[]={0x05};
                        _openData=[[NSData alloc] initWithBytes:byte length:sizeof(byte)];
                    }
                    // DBay-EEGS使用这个命令
                    else if([deviceName hasPrefix:@"Dbay-EEGS"]){
                        Byte byte[]={0x04,0x01,0x01};
                        _openData=[[NSData alloc] initWithBytes:byte length:sizeof(byte)];
                    }
                    [peripheral writeValue:_openData forCharacteristic:character type:CBCharacteristicWriteWithResponse];
                    
                    NSMutableDictionary * _infoDict=[NSMutableDictionary new];
                    [_infoDict setObject:peripheral forKey:@"peripheral"];
                    [_infoDict setObject:character forKey:@"character"];
                    [_weakSelf.notifyState setObject:_infoDict forKey:peripheral.identifier.UUIDString];
                });
            }
        } else if ([character.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC2_UUID]]){
            [peripheral setNotifyValue:YES forCharacteristic:character];
        } else if ([character.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC3_UUID]]){
            [peripheral setNotifyValue:YES forCharacteristic:character];
        } else if ([character.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC4_UUID]]){
            [peripheral setNotifyValue:YES forCharacteristic:character];
        } else if ([character.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC5_UUID]]){
            [peripheral setNotifyValue:YES forCharacteristic:character];
        }
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(nonnull CBCharacteristic *)characteristic error:(nullable NSError *)error
{
    if (error) {
        NSLog(@"错误didUpdateNotification：%@",error);
        return;
    }
    
    CBCharacteristicProperties properties = characteristic.properties;
    if (properties & CBCharacteristicPropertyRead) {
        //如果具备读特性，即可以读取特性的value
        [peripheral readValueForCharacteristic:characteristic];
    }
}

-(int)getDeviceIndex:(NSString *)address
{
    int index=-1;
    for(int i=0;i<_allConnectedDevice.count;i++){
        NSDictionary * _dict=[_allConnectedDevice objectAtIndex:i];
        CBPeripheral * _per=[_dict objectForKey:@"peripheral"];
        if([_per.identifier.UUIDString isEqualToString:address]){
            index=[[_dict objectForKey:@"index"] intValue];
            return index;
        }
    }
    return index;
}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    NSData *data = characteristic.value;
    NSString * _string=[self hexStringFromData:data];
    if([characteristic.UUID.UUIDString isEqualToString:CHARACTERISTIC2_UUID]){
        if([[DBAlgorithmCalculate sharedInstance] isCanUseSDK]){
            int _deviceIndex=[self getDeviceIndex:peripheral.identifier.UUIDString];
            __weak __typeof(self) _weakSelf = self;
            [[DBAlgorithmCalculate sharedInstance] parseRawString:_string deviceIndex:_deviceIndex eegCompletion:^(DBEegModel * eeg) {
                if([eeg getErrorCode]==DBErrorCodeSuccess){
                    NSMutableDictionary * _perDict=[[NSMutableDictionary alloc] initWithDictionary:[_weakSelf.dataCache objectForKey:peripheral.identifier.UUIDString]];
                    NSMutableArray * _leftArray=[[NSMutableArray alloc] initWithArray:[_perDict objectForKey:@"left"]];
                    NSMutableArray * _rightArray=[[NSMutableArray alloc] initWithArray:[_perDict objectForKey:@"right"]];
                    [_leftArray addObjectsFromArray:eeg.fp7Eegs];
                    [_rightArray addObjectsFromArray:eeg.fp8Eegs];
                    [_perDict setObject:_leftArray forKey:@"left"];
                    [_perDict setObject:_rightArray forKey:@"right"];
                    [_weakSelf.dataCache setObject:_perDict forKey:peripheral.identifier.UUIDString];
                }
            } completion:^(DBAlgorithmModel * model) {
                /*
                 信号质量的处理
                 */
                int _signalLeft=[model getSignalQuailty:DBSignalQuailtyTypeLeft];
                int _signalRight=[model getSignalQuailty:DBSignalQuailtyTypeRight];
//                NSLog(@"signal is:%d,%d",_signalLeft,_signalRight);
                int _signal=0;
                if(_signalLeft==0&&_signalRight==0){
                    _signal=0;// 两个都戴好
                } else if (_signalLeft==1||_signalRight==1){
                    _signal=-1;// 两个都不戴好（脱落）
                } else {
                    _signal=1; // 接触不良
                }

                NSDictionary * _deviceDict = @{
                    @"address": peripheral.identifier.UUIDString,
                    @"name": peripheral.name,
                    @"type":[NSNumber numberWithInt:type_data],
                };
                NSMutableDictionary * _algorithmDict=[[NSMutableDictionary alloc] initWithDictionary:_deviceDict];

                if([model getErrorCode]==DBErrorCodeSuccess){// 只有code为success的时候才能获取tbr指标
                    NSLog(@"tbr is:%.2f",[model getTbr]);
                    
                    /**
                     能量值的获取
                     */
                    DBAlgorithmEnergyModel * _energyModel=[model getEnergy];
                    
                    [_algorithmDict setObject:[NSString stringWithFormat:@"%d",[model getIndexWa]] forKey:@"focus"];
                    [_algorithmDict setObject:[NSString stringWithFormat:@"%.4f",[model getIndexAsy]] forKey:@"emotion"];
                    [_algorithmDict setObject:[NSString stringWithFormat:@"%d",[model getIndexFatig]] forKey:@"fatigue"];
                    [_algorithmDict setObject:[NSString stringWithFormat:@"%d",[model getIndexStress]] forKey:@"stress"];
                    NSMutableDictionary * _energys=[NSMutableDictionary new];
                    [_energys setObject:[NSString stringWithFormat:@"%.4f",[_energyModel getEnergyType:DBAlgorithmEnergyTypeAlpha]] forKey:@"alpha"];
                    [_energys setObject:[NSString stringWithFormat:@"%.4f",[_energyModel getEnergyType:DBAlgorithmEnergyTypeBeta]] forKey:@"beta"];
                    [_energys setObject:[NSString stringWithFormat:@"%.4f",[_energyModel getEnergyType:DBAlgorithmEnergyTypeTheta]] forKey:@"theta"];
                    [_energys setObject:[NSString stringWithFormat:@"%.4f",[_energyModel getEnergyType:DBAlgorithmEnergyTypeDelta]] forKey:@"delta"];
                    [_energys setObject:[NSString stringWithFormat:@"%.4f",[_energyModel getEnergyType:DBAlgorithmEnergyTypeGamma]] forKey:@"gamma"];
                    [_algorithmDict setObject:_energys forKey:@"energy"];
                    [_algorithmDict setObject:[NSString stringWithFormat:@"%.4f",[model getTbr]] forKey:@"mindfulness"];
                }
                [_algorithmDict setObject:[NSNumber numberWithInt:_signal] forKey:@"signal"];
                if([[_weakSelf.rawInfo objectForKey:peripheral.identifier.UUIDString] boolValue]){
                    NSMutableDictionary * _perDict=[[NSMutableDictionary alloc] initWithDictionary:[_weakSelf.dataCache objectForKey:peripheral.identifier.UUIDString]];
                    NSMutableArray * _leftArray=[[NSMutableArray alloc] initWithArray:[_perDict objectForKey:@"left"]];
                    NSMutableArray * _rightArray=[[NSMutableArray alloc] initWithArray:[_perDict objectForKey:@"right"]];
                    [_algorithmDict setObject:[NSArray arrayWithArray:_leftArray] forKey:@"left"];
                    [_algorithmDict setObject:[NSArray arrayWithArray:_rightArray] forKey:@"right"];
                    [_leftArray removeAllObjects];
                    [_rightArray removeAllObjects];
                    [_perDict setObject:_leftArray forKey:@"left"];
                    [_perDict setObject:_rightArray forKey:@"right"];
                    [_weakSelf.dataCache setObject:_perDict forKey:peripheral.identifier.UUIDString];
                }

                [self sendToApp:[self objectToJson:_algorithmDict]];
            }];
        }
    } else if([characteristic.UUID.UUIDString isEqualToString:CHARACTERISTIC3_UUID]){
        NSMutableDictionary * _perDict=[[NSMutableDictionary alloc] initWithDictionary:[_notifyState objectForKey:peripheral.identifier.UUIDString]];
        if(_string.length==8*2){
            int _index=([[_string substringWithRange:NSMakeRange(0, 1)] intValue]*16+[[_string substringWithRange:NSMakeRange(1, 1)] intValue]);
            if(_index==129){
                NSString * _versionStr=@"";
                for(int i=5;i<8;i++){
                    int _index1=[[_string substringWithRange:NSMakeRange(2*i, 1)] intValue];
                    int _index2=[[_string substringWithRange:NSMakeRange(2*i+1, 1)] intValue];
                    if([_versionStr isEqualToString:@""]){
                        _versionStr=[NSString stringWithFormat:@"%d",_index1*16+_index2];
                    } else {
                        _versionStr=[NSString stringWithFormat:@"%@.%d",_versionStr,_index1*16+_index2];
                    }
                }
                NSDictionary * deviceDict = @{
                    @"address": peripheral.identifier.UUIDString,
                    @"name":peripheral.name,
                    @"version":_versionStr,
                    @"type":[NSNumber numberWithInt:type_get_device_info],
                };
                [self sendToApp:[self objectToJson:deviceDict]];
            }
        } else if(_string.length==2*2){
            int _index=([[_string substringWithRange:NSMakeRange(0, 1)] intValue]*16+[[_string substringWithRange:NSMakeRange(1, 1)] intValue]);
            if(_index==130){
                int _index1=[[_string substringWithRange:NSMakeRange(2, 1)] intValue];
                int _index2=[[_string substringWithRange:NSMakeRange(3, 1)] intValue];
                NSNumber * _batteryValue=[NSNumber numberWithInt:_index1*16+_index2];
                
                NSDictionary * deviceDict = @{
                    @"address": peripheral.identifier.UUIDString,
                    @"name":peripheral.name,
                    @"battery":_batteryValue,
                    @"type":[NSNumber numberWithInt:type_get_device_info],
                };
                [self sendToApp:[self objectToJson:deviceDict]];
            }
        }
    }
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(nullable NSError *)error
{
    NSLog(@"设备已经断开连接");
    [_notifyState removeObjectForKey:peripheral.identifier.UUIDString];
    [_allConnectedDevice enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        CBPeripheral * _per=[obj objectForKey:@"peripheral"];
        if([_per.identifier.UUIDString isEqualToString:peripheral.identifier.UUIDString]){
            [[DBAlgorithmCalculate sharedInstance] deleteDevice:_per.name];
            [_allConnectedDevice removeObjectAtIndex:idx];
            *stop=YES;
        }
    }];
    NSDictionary * deviceDict = @{
        @"address": peripheral.identifier.UUIDString,
        @"name": peripheral.name,
        @"type":[NSNumber numberWithInt:type_disconnected],
        @"state": [NSNumber numberWithInt:(int)peripheral.state],
    };
    [self sendToApp:[self objectToJson:deviceDict]];
}

- (NSString *)hexStringFromData:(NSData *)string{
    Byte *bytes = (Byte *)[string bytes]; //下面是Byte 转换为16进制。
    NSString *hexStr=@""; for(int i=0;i<[string length];i++) {
        NSString *newHexStr = [NSString stringWithFormat:@"%X",bytes[i]&0xFF];///16进制数
        if([newHexStr length]==1)
            hexStr = [NSString stringWithFormat:@"%@0%@",hexStr,newHexStr];
        else
            hexStr = [NSString stringWithFormat:@"%@%@",hexStr,newHexStr];
    }
    return hexStr;
}


extern "C"
{
    void init(char* param)
    {
        [[BLHelper sharedInstance] init:getString(param)];
    }

    bool isAvailable()
    {
        return [[BLHelper sharedInstance] isAvailable];
    }

    void scanDevice()
    {
        [[BLHelper sharedInstance] scanDevice];
    }

    void scanStop()
    {
        [[BLHelper sharedInstance] stopScan];
    }

    void connectDevice(char* address)
    {
        [[BLHelper sharedInstance] connectDevice:getString(address)];
    }

    void disconnectDevice(char* address)
    {
        [[BLHelper sharedInstance] disconnectDevice:getString(address)];
    }
    
    void openWriteRaw(char* address)
    {
        [[BLHelper sharedInstance] openWriteRaw:getString(address)];
    }
    
    void getVersion(char * address)
    {
        [[BLHelper sharedInstance] getVersion:getString(address)];
    }
    
    void getBattery(char * address)
    {
        [[BLHelper sharedInstance] getBattery:getString(address)];
    }
}

@end
