## Unity iOS SDK接入说明

### SDK文件
下载SDK，[alg-unity-ios-v1.2.zip](alg-unity-ios-v1.2.zip)

```
1、DBAlgorithm.framework //包含算法SDK的动态库文件
2、BLHelper.h BLHelper.m  //调用桥接SDK文件的类
3、README.md //SDK接入说明文件
```

### 对应桥接类名

发送数据给unity，receiver = "BLHelper", method = "onMsg"

```
/// 发送所有的通信数据到unity端
/// @param json 结构化的数据
-(void)sendToApp:(NSString*)json
{
    // PS:当使用unity3d的时候，使用下面的方法发送消息，同时需要打开"UnityInterface.h"文件的引用
    UnitySendMessage(getChar(receiver), getChar(method), getChar(json));
}
```
扫描设备实例
```
NSDictionary * deviceDict = @{
    @"address": peripheral.identifier.UUIDString,
    @"name": peripheral.name,
    @"state": [NSNumber numberWithInt:(int)peripheral.state],
    @"type":[NSNumber numberWithInt:type_connect_succeed],
};
[self sendToApp:[self objectToJson:deviceDict]];
```


### SDK 方法说明
1. init(String param); 初始化数据和请求蓝牙权限.
2. scanDevice(); 扫描设备.
3. scanStop(); 停止扫描.
4. connectDevice(String address); 连接设备.
5. disconnectDevice(String address); 断开连接.
6. getVersion(String address); 获取设备固件版本.
7. getBattery(String address); 获取设备电量.

### unity接收方法数据方法

```
enum TYPE {
    none = 0,
    data = 1,//蓝牙数据
    find_device = 2,//扫描设备回调
    connect_succeed = 3,//连接成功回调
    connect_failed = 4,//连接失败回调
    disconnected = 5,//断开连接回调
    device = 6 //获取设备信息（电量，固件版本）
};
```
1、蓝牙数据参数
（signal 0信号质量好，1信号质量异常）
```
{
    "address": "D6:5E:71:12:99:0C",
    "type": 1,
    "mindfulness": 0,
    "stress": 0,
    "fatigue": 0,
    "emotion": 0,
    "focus": 0,
    "energy": {
        "alpha": 0,
        "beta": 0,
        "theta": 0,
        "delta": 0,
        "gamma": 0
    },
    "signal": 1
}
```
2、扫描设备返回数据
（state 0未连接, 1连接中, 2已连接）
```
{
    "type": 2,
    "name": "Dbay-EEG2-283D",
    "address": "D2:F7:33:DA:C8:C6",
    "state": 0
}
```
3、连接设备返回数据

```
{
    "type": 3,
    "address": "DF:EB:68:47:64:30",
    "name": "Dbay-EEG2-283D",
    "state": 2
}
```
4、连接失败返回数据

```
{
    "type": ,
    "address": "DF:EB:68:47:64:30",
    "name": "Dbay-EEG2-283D",
    "state": 0
}
```
5、断开设备返回数据

```
{
    "type": 5,
    "address": "DF:EB:68:47:64:30",
    "name": "Dbay-EEG2-283D",
    "state": 0
}
```
6、获取设备电量返回数据

```
{
    "type": 6,
    "address": "DF:EB:68:47:64:30",
    "name": "Dbay-EEG2-283D",
    "battery": 60
}
```
7、获取设备版本返回数据

```
{
    "type": 6,
    "address": "DF:EB:68:47:64:30",
    "name": "Dbay-EEG2-283D",
    "version": "0.2.9"
}
```


Unity接收扫描设备数据
```
void onMsg(string json)
{
    Debug.Log("---onMsg---" + json);
    BLData data = JsonUtility.FromJson<BLData>(json);
    if (data.type == (int)TYPE.find_device)
    {
        if (!blDevice.ContainsKey(data.address))
        {
            blDeviceList.Add(data);
            blDevice.Add(data.address, data);
        }
        else
            blDevice[data.address] = data;
            refreshDevice();
#if USE_MESSENGER
        Messenger.Broadcast("type_find_device", data);
#endif
    }
}
```
